# Note: this file has been initially generated by Module::Build::Convert 0.49

use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new
  (
   module_name => 'Crashplan::Client',
   dist_author => q{Arnaud (Arhuman) ASSAD <arnaud.assad@jaguar-network.com>},
   dist_version_from => 'lib/Crashplan/Client.pm',
   requires => {
                 'Carp' => 0,
                 'JSON' => 0,
                 'MIME::Base64' => 0,
                 'REST::Client' => 0,
                 'Test::More' => 0
               },
   add_to_cleanup => [
                       'Crashplan-Client-*'
                     ],
   license => 'perl',
   create_makefile_pl => 'traditional',
  );

$build->create_build_script;
